#include "exceptions.h"
#include "usluge.h"
#include <iostream>
using namespace std;

Usluga::Usluga(const Broj& broj_od, const Broj& broj_ka) : broj_od_(broj_od), broj_ka_(broj_ka) {}

void Usluga::pisi() const {
	broj_od_.pisi();
	cout << " -> ";
	broj_ka_.pisi();
}

const int Poziv::uspostavljanje_veze_domaci_saobracaj = 0;
const int Poziv::uspotavljanje_veze_inostranstvo = 30;
const int Poziv::zapoceti_minut_domaci_saobracaj = 10;
const int Poziv::zapoceti_minut_inostranstvo = 50;

// Bez inicijalizacije nasledjenog podobjekta, prevodilac inicijalizuje podobjekat podrazumevanim konstruktorom osnovne klase
// Kako klasa Usluga nema podrazumevani konstruktor, program se ne bi uspesno preveo
Poziv::Poziv(const Broj& broj_od, const Broj& broj_ka, int trajanje) : Usluga(broj_od, broj_ka), trajanje_(trajanje) {}

double Poziv::cena() const {
	if (trajanje_ == 0) return 0;
	if (broj_od_.istaDrzava(broj_ka_)) {
		return uspostavljanje_veze_domaci_saobracaj + ((trajanje_ / 60) + 1) * zapoceti_minut_domaci_saobracaj;
	}
	else {
		return uspotavljanje_veze_inostranstvo + ((trajanje_ / 60) + 1) * zapoceti_minut_inostranstvo;
	}
}

void Poziv::pisi() const {
	Usluga::pisi();
	cout << " (" << trajanje_ / 60 << ":" << trajanje_ % 60 << ")";
}

const int Poruka::slanje_poruke_domaci_saobracaj = 3;
const int Poruka::slanje_poruke_inostranstvo = 20;

Poruka::Poruka(const Broj& broj_od, const Broj& broj_ka, const string& tekst) : Usluga(broj_od, broj_ka), tekst_(tekst) {
	// U slucaju da je jedan od brojeva fiksni, objekat Poruke ne treba da se kreira
	// U tom slucaju bacamo izuzetak
	// Izuzetak bacen u konstruktoru automatski unistava ceo objekat i sve delove koji su do tog trenutka inicijalizovani
	// U slucaju da smo do ovog trenutka napravili nesto u dinamickoj memoriji, morali bismo taj podatak da izbrisemo u try/catch naredbi i prosledimo izuzetak dalje
	
	// int* podatak = nullptr;
	// try {
		// podatak = new int;
		if (broj_od_.fiksni() || broj_ka_.fiksni()) {
			throw NeuspesnoPoslataPoruka("Nije moguce uspostaviti uslugu poruke ukoliko je jedan od brojeva tipa fiksni.");
		}
		// ...
		// delete podatak;
	// }
	// catch (NeuspesnoPoslataPoruka& e) {
		// delete podatak;
		// throw;
	// }
}

double Poruka::cena() const {
	if (broj_od_.istaDrzava(broj_ka_)) {
		return slanje_poruke_domaci_saobracaj;
	}
	else {
		return slanje_poruke_inostranstvo;
	}
}

void Poruka::pisi() const {
	Usluga::pisi();
	cout << " " << tekst_;
}
